package paraphylo.parser;


import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import paraphylo.dao.Parameters;
import paraphylo.dataset.GeneFamily;
import paraphylo.dataset.MultiGeneFamilySpeciesDataSet;


public class Parser {
	
	public static MultiGeneFamilySpeciesDataSet parseTriples(Parameters parameters){
		String filename = parameters.getInput();
		File file = new File (filename);
		
		if (!file.exists()){
			return null;
		}
		
		try {
			BufferedReader br = new BufferedReader(new FileReader(file));
			int noOfSpecies = 0;
			String[] speciesNames = new String[0];
			String speciesTree = "";
			
			String line;
			while ((line=br.readLine())!=null){
				line=line.trim();
				if (line.contentEquals("#NoOfSpecies")){
					line = br.readLine().trim();
					noOfSpecies = Integer.parseInt(line);
					speciesNames = new String[noOfSpecies];
				}
				else if (line.contentEquals("#SpeciesTree")){
					line = br.readLine().trim();
					Pattern pattern = Pattern.compile("[^(,);]+");
					Matcher matcher = pattern.matcher(line);
					int counter=0;
					while (matcher.find()) {
						speciesNames[counter] = matcher.group();
						counter++;
				    }
					speciesTree = line;
					break;
				}
				else if (line.contentEquals("#SpeciesNames")){
					line = br.readLine().trim();
					speciesNames = line.split("\\s");
					break;
				}
			}

			MultiGeneFamilySpeciesDataSet ds = new MultiGeneFamilySpeciesDataSet(noOfSpecies, speciesNames);
			ds.setOriginalTree(speciesTree);
			
			while ((line=br.readLine())!=null){
				line=line.trim();
				if (line.contentEquals("#Triples")){
					Pattern pattern = Pattern.compile("^[0-9]+,[0-9]+\\|[0-9]+\\s[0-9\\.E-]+$");
					while ((line=br.readLine())!=null){
						line=line.trim();
						Matcher matcher = pattern.matcher(line);
						if (matcher.find()){
							String[] values = matcher.group().split("[,\\|\\s]");
							int a = Integer.parseInt(values[0]);
							int b = Integer.parseInt(values[1]);
							int c = Integer.parseInt(values[2]);
							double n = Double.parseDouble(values[3]);
							ds.setTriple(a, b, c, n);
						}
						else{
							break;
						}
					}
				}
			}
			br.close();
			return ds;
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}
	
	public static MultiGeneFamilySpeciesDataSet parseCographs(Parameters parameters){
		String filename = parameters.getInput();
		File file = new File (filename);
		
		if (!file.exists()){
			return null;
		}
		
		try {
			BufferedReader br = new BufferedReader(new FileReader(file));
			int noOfSpecies = 0;
			String[] speciesNames = new String[0];
			String speciesTree = "";
			
			String line;
			while ((line=br.readLine())!=null){
				line=line.trim();
				if (line.contentEquals("#NoOfSpecies")){
					line = br.readLine().trim();
					noOfSpecies = Integer.parseInt(line);
					speciesNames = new String[noOfSpecies];
				}
				else if (line.contentEquals("#SpeciesTree")){
					line = br.readLine().trim();
					Pattern pattern = Pattern.compile("[^(,);]+");
					Matcher matcher = pattern.matcher(line);
					int counter=0;
					while (matcher.find()) {
						speciesNames[counter] = matcher.group();
						counter++;
				    }
					speciesTree = line;
					break;
				}
				else if (line.contentEquals("#SpeciesNames")){
					line = br.readLine().trim();
					speciesNames = line.split("\\s");
					break;
				}
			}

			MultiGeneFamilySpeciesDataSet ds = new MultiGeneFamilySpeciesDataSet(noOfSpecies, speciesNames);
			ds.setOriginalTree(speciesTree);
			
			int noOfGenes = 0;
			GeneFamily geneFamily = null;
			while ((line=br.readLine())!=null){
				line=line.trim();
				if (line.startsWith("#GeneFamily")){
					noOfGenes = 0;
					geneFamily = new GeneFamily(noOfSpecies, speciesNames);
				}
				else if (line.contentEquals("#GeneSpeciesAssociations")){
					line = br.readLine().trim();
					String associations[] = line.split("\\s+");
					noOfGenes = associations.length;
					String[] geneSpeciesAssociation = new String[noOfGenes];
					for (int i=0; i<noOfGenes; i++){
						geneSpeciesAssociation[i] = associations[i];
					}
					geneFamily.init(geneSpeciesAssociation);
					ds.addGeneFamilies(geneFamily);
				}
				else if ( line.contentEquals("#Cograph")){
					if (parameters.getCographSize()<=0 || noOfGenes<=parameters.getCographSize()){
						Boolean[][] cograph = new Boolean[noOfGenes][noOfGenes];
						for (int i=0; i<noOfGenes; i++){
							line=br.readLine().trim();
							String orthology[] = line.split("\\s+");
							for (int j=0; j<noOfGenes; j++){
								Boolean value = new Boolean(Double.parseDouble(orthology[j])>=0.5);
								cograph[i][j]=value;
							}
						}
						geneFamily.addCograph(cograph);
					}
				}
			}
			br.close();
			return ds;
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}
	
	public static MultiGeneFamilySpeciesDataSet parseDs(Parameters parameters){
		
		String filename = parameters.getInput();
		File file = new File (filename);
		
		if (!file.exists()){
			return null;
		}
		
		try {
			BufferedReader br = new BufferedReader(new FileReader(file));
			int noOfSpecies = 0;
			String[] speciesNames = new String[0];
			String speciesTree = "";
			
			String line;
			while ((line=br.readLine())!=null){
				line=line.trim();
				if (line.contentEquals("#NoOfSpecies")){
					line = br.readLine().trim();
					noOfSpecies = Integer.parseInt(line);
					speciesNames = new String[noOfSpecies];
				}
				else if (line.contentEquals("#SpeciesTree")){
					line = br.readLine().trim();
					Pattern pattern = Pattern.compile("[^(,);]+");
					Matcher matcher = pattern.matcher(line);
					int counter=0;
					while (matcher.find()) {
						speciesNames[counter] = matcher.group();
						counter++;
				    }
					speciesTree = line;
					break;
				}
				else if (line.contentEquals("#SpeciesNames")){
					line = br.readLine().trim();
					speciesNames = line.split("\\s");
					break;
				}
			}
			
			MultiGeneFamilySpeciesDataSet ds = new MultiGeneFamilySpeciesDataSet(noOfSpecies, speciesNames);
			ds.setOriginalTree(speciesTree);
			
			int noOfGenes = 0;
			String[] geneSpeciesAssociation = new String[0];
			GeneFamily geneFamily = null;

			while ((line=br.readLine())!=null){
				line=line.trim();
				if (line.startsWith("#GeneFamily")){
					noOfGenes = 0;
					geneFamily = new GeneFamily(noOfSpecies, speciesNames);;
				}
				else if (line.contentEquals("#GeneTree")){
					line = br.readLine().trim();
					String geneTree = line;
					geneFamily.setGeneTree(geneTree);
					if (geneFamily.getGeneNames()==null || geneFamily.getGeneNames().equals("")){
						Pattern pattern = Pattern.compile("[^(,);]+");
						Matcher matcher = pattern.matcher(line);
						List<String> geneNames = new ArrayList<String>();
						while (matcher.find()) {
							geneNames.add(matcher.group());
					    }
						geneFamily.setGeneNames(geneNames.toArray(new String[0]));
					}
				}
				else if (line.contentEquals("#GeneNames")){
					line = br.readLine().trim();
					String[] geneNames = line.split("\\s+");
					geneFamily.setGeneNames(geneNames);
				}
				else if (line.contentEquals("#GeneSpeciesAssociations")){
					line = br.readLine().trim();
					String associations[] = line.split("\\s+");
					noOfGenes = associations.length;
					if (parameters.getCographSize()<=0 || noOfGenes<=parameters.getCographSize()){
						geneSpeciesAssociation = new String[noOfGenes];
						for (int i=0; i<noOfGenes; i++){
							geneSpeciesAssociation[i] = associations[i];
							// add association noise
							if (parameters.noiseAssociations() && parameters.nextNoise()){
								geneSpeciesAssociation[i] = speciesNames[parameters.getRandom().nextInt(noOfSpecies)];
							}
						}
						geneFamily.init(geneSpeciesAssociation);
						ds.addGeneFamilies(geneFamily);
					}
				}
				else if ( line.contentEquals("#OrthologyMatrix" )){
					if (parameters.getCographSize()<=0 || noOfGenes<=parameters.getCographSize()){
						double[][] orthologyMatrix = new double[noOfGenes][noOfGenes];
						for (int i=0; i<noOfGenes; i++){
							line=br.readLine().trim();
							String orthology[] = line.split("\\s+");
							for (int j=0; j<noOfGenes; j++){
								double value = Double.parseDouble(orthology[j]);
								// add noise to orthology matrix
								if (i!=j && !geneSpeciesAssociation[i].equals(geneSpeciesAssociation[j]) && ( (parameters.noiseOrthologs() && value==1d) || (parameters.noiseParalogs() && value==0d) ) ){
									value = noise(value>0.5, parameters);
								}
								orthologyMatrix[i][j]=value;
							}
						}
						geneFamily.setOrthologyMatrix(orthologyMatrix);
					}
				}
				else if ( line.contentEquals("#Bitscores" )){
					if (parameters.getCographSize()<=0 || noOfGenes<=parameters.getCographSize()){
						double[][] bitscoreMatrix = new double[noOfGenes][noOfGenes];
						for (int i=0; i<noOfGenes; i++){
							line=br.readLine().trim();
							String orthology[] = line.split("\\s+");
							for (int j=0; j<noOfGenes; j++){
								double value = Double.parseDouble(orthology[j]);
								bitscoreMatrix[i][j]=value;
							}
						}
						geneFamily.setBitscores(bitscoreMatrix);
					}
				}
				else if ( line.contentEquals("#E-values" )){
					if (parameters.getCographSize()<=0 || noOfGenes<=parameters.getCographSize()){
						double[][] evalueMatrix = new double[noOfGenes][noOfGenes];
						for (int i=0; i<noOfGenes; i++){
							line=br.readLine().trim();
							String orthology[] = line.split("\\s+");
							for (int j=0; j<noOfGenes; j++){
								double value = Double.parseDouble(orthology[j]);
								evalueMatrix[i][j]=value;
							}
						}
						geneFamily.setEvalues(evalueMatrix);
					}
				}
			}
			br.close();
			return ds;
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}
	
	private static double noise(boolean value, Parameters parameters){
		double x = parameters.getRandom().nextDouble();
		if (x<parameters.getNoiseProbability()){
			return value?0d:1d;
		}
		return value?1d:0d;
	}
}
