package paraphylo.util;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import paraphylo.dao.Triple;


public class ClosureComputation {

	public static List<Triple> getClosure(List<Triple> triples){
		if (!checkTripleConsistency(triples)){
			return null;
		}
		int noOfGenes = 0;
		for (Triple triple : triples){
			noOfGenes = Math.max(noOfGenes, Math.max(triple.getA(), Math.max(triple.getB(), triple.getC())));
		}
		noOfGenes++;
		boolean occurrences[][][] = new boolean[noOfGenes][noOfGenes][noOfGenes];
		for (Triple triple : triples){
			int x = Math.min(triple.getA(), Math.min(triple.getB(), triple.getC()));
			int y = 0;
			int z = Math.max(triple.getA(), Math.max(triple.getB(), triple.getC()));
			if (x<triple.getA() && triple.getA()<z){
				y = triple.getA();
			}
			else if (x<triple.getB() && triple.getB()<z){
				y = triple.getB();
			}
			else if (x<triple.getC() && triple.getC()<z){
				y = triple.getC();
			}
			occurrences[x][y][z] = true;
		}
		List<Triple> newTriples = new ArrayList<Triple>();
		for (int x=0; x<noOfGenes; x++){
			for (int y=x+1; y<noOfGenes; y++){
				for (int z=y+1; z<noOfGenes; z++){
					if (!occurrences[x][y][z]){
						boolean consistent[] = new boolean[3];
						Triple triple0 = new Triple(x,y,z);
						Triple triple1 = new Triple(x,z,y);
						Triple triple2 = new Triple(y,z,x);
						Triple candidates[] = {triple0, triple1, triple2};
						for (int i=0; i<3; i++){
							triples.add(candidates[i]);
							consistent[i] = checkTripleConsistency(triples);
							triples.remove(candidates[i]);
						}
						if (consistent[0] && !consistent[1] && !consistent[2]){
							newTriples.add(candidates[0]);
						}
						if (!consistent[0] && consistent[1] && !consistent[2]){
							newTriples.add(candidates[1]);
						}
						if (!consistent[0] && !consistent[1] && consistent[2]){
							newTriples.add(candidates[2]);
						}
					}
				}
			}
		}
		newTriples.addAll(triples);
		return newTriples;
	}
	
	private static boolean checkTripleConsistency(List<Triple> triples){
		if (triples.size()<2){
			return true;
		}
		Set<Set<Integer>> sets = new HashSet<Set<Integer>>();
		for (Triple triple : triples){
			Set<Integer> set1 = new HashSet<Integer>();
			set1.add(triple.getA());
			set1.add(triple.getB());
			for (Set<Integer> s : sets){
				if (s.contains(triple.getA())){
					set1.addAll(s);
					sets.remove(s);
					break;
				}
			}
			for (Set<Integer> s : sets){
				if (s.contains(triple.getB())){
					set1.addAll(s);
					sets.remove(s);
					break;
				}
			}
			sets.add(set1);
			boolean found2 = false;
			for (Set<Integer> s : sets){
				if (s.contains(triple.getC())){
					found2=true;
					break;
				}
			}
			if (!found2){
				Set<Integer> set2 = new HashSet<Integer>();
				set2.add(triple.getC());
				sets.add(set2);
			}
		}
		if (sets.size()<2){
			return false;
		}
		for (Set<Integer> set : sets){
			List<Triple> newTriples = new ArrayList<Triple>();
			for (Triple triple : triples){
				if (set.contains(triple.getA()) && set.contains(triple.getB()) && set.contains(triple.getC())){
					newTriples.add(triple);
				}
			}
			if (!checkTripleConsistency(newTriples)){
				return false;
			}
		}
		return true;
	}
}
