package paraphylo.util;

import java.util.Collection;

import paraphylo.dataset.GeneFamily;
import paraphylo.dataset.MultiGeneFamilySpeciesDataSet;

public class Utils {

	public static int countOccurrences(String haystack, char needle)
	{
	    int count = 0;
	    for (int i=0; i < haystack.length(); i++)
	    {
	        if (haystack.charAt(i) == needle)
	        {
	             count++;
	        }
	    }
	    return count;
	}
	
	public static double computeSubtreeSupport(Collection<Integer> subSpecies,
			double[][][] allTriples, int noSpecies) {
		double used=0;
		double sum=0;
		Integer[] speciesArray = subSpecies.toArray(new Integer[0]);
		for (int i=0; i<speciesArray.length; i++){
			int a = speciesArray[i];
			for (int j=i+1; j<speciesArray.length; j++){
				int b = speciesArray[j];
				if (a!=b){
					for (int c=0; c<noSpecies; c++){
						if (!subSpecies.contains(c)){
							used += allTriples[a<b?a:b][a<b?b:a][c];
							sum += allTriples[a<b?a:b][a<b?b:a][c] + allTriples[a<c?a:c][a<c?c:a][b] + allTriples[b<c?b:c][b<c?c:b][a];
						}
					}
				}
			}
		}
		if (sum > 0){
			return used/sum;
		}
		return Double.NaN;
	}
	
	public static double computeSupport(MultiGeneFamilySpeciesDataSet ds) {
		double used = 0;
		double sum = 0;
		for (int a=0; a<ds.getNoOfSpecies(); a++){
			for (int b=a+1; b<ds.getNoOfSpecies(); b++){
				for (int c=0; c<ds.getNoOfSpecies(); c++){
					if (a!=c && b!=c && ds.getConsistentTriple(a, b, c)){
						used += ds.getTriple(a, b, c);
						sum += ds.getTriple(a, b, c) + ds.getTriple(a<c?a:c, a<c?c:a, b) + ds.getTriple(b<c?b:c, b<c?c:b, a);
					}
				}
			}
		}
		if (sum > 0){
			return used/sum;
		}
		return Double.NaN;
	}
	
	public static boolean isCograph(GeneFamily gf) {
		int noGenes = gf.getNoOfGenes();
		double[][] orthology = gf.getOrthologyMatrix();
		for (int a=0; a<noGenes-1; a++){
			for (int b=0; b<noGenes; b++){
				if (a!=b && orthology[a][b]>0.5){
					for (int c=0; c<noGenes; c++){
						if (a!=c && b!=c && orthology[b][c]>0.5 && orthology[a][c]<0.5){
							for (int d=a+1; d<noGenes; d++){
								if (b!=d && c!=d && orthology[c][d]>0.5 && orthology[a][d]<0.5 && orthology[b][d]<0.5){
									return false;
								}
							}
						}
					}
				}
			}
		}
		return true;
	}
	
	public static boolean checkOrthology(GeneFamily gf) {
		int noGenes = gf.getNoOfGenes();
		double[][] orthology = gf.getOrthologyMatrix();
		for (int a=0; a<noGenes; a++){
			if (orthology[a][a] > 0.5){
				return false;
			}
			for (int b=a+1; b<noGenes; b++){
				// check symmetry
				if (orthology[a][b]!=orthology[b][a]){
					return false;
				}
				// check if all orthologs are between different species
				if (orthology[a][b] > 0.5 && gf.getGeneSpeciesAssociation()[a].equals(gf.getGeneSpeciesAssociation()[b])){
					return false;
				}
			}
		}
		return true;
	}
	
	public static boolean existsInterSpeciesParalogyInCograph(GeneFamily gf, Boolean[][] cograph) {
		int noGenes = gf.getNoOfGenes();
		for (int a=0; a<noGenes-1; a++){
			for (int b=a+1; b<noGenes; b++){
				// check if there exists paralogs between different species
				if (!cograph[a][b] && !gf.getGeneSpeciesAssociation()[a].equals(gf.getGeneSpeciesAssociation()[b])){
					return true;
				}
			}
		}
		return false;
	}
}
