package paraphylo.converter;

import java.util.HashMap;
import java.util.Map;
import java.util.Random;

public class GeneFamily {

	String[] species;
	String[] genes;
	Map<String,Integer> geneIndex;
	double[][] predictedOrthology;
	double[][] blastOrthology;
	double[][] bitscore;
	double[][] evalue;
	int noOfGenes;
	
	public GeneFamily(String[] species, String[] genes) {
		super();
		this.species = species;
		this.genes = genes;
		this.noOfGenes = genes.length;
		this.geneIndex = new HashMap<String,Integer>();
		for (int i=0; i<genes.length; i++){
			this.geneIndex.put(genes[i], i);
		}
		this.predictedOrthology = new double[noOfGenes][noOfGenes];
		this.blastOrthology = new double[noOfGenes][noOfGenes];
		this.bitscore = new double[noOfGenes][noOfGenes];
		this.evalue = new double[noOfGenes][noOfGenes];
	}

	public String[] getSpecies() {
		return species;
	}

	public void setSpecies(String[] species) {
		this.species = species;
	}

	public String[] getGenes() {
		return genes;
	}

	public void setGenes(String[] genes) {
		this.genes = genes;
	}

	public double[][] getPredictedOrthology() {
		return predictedOrthology;
	}

	public void setPredictedOrthology(double[][] predictedOrthology) {
		this.predictedOrthology = predictedOrthology;
	}

	public double[][] getBlastOrthology() {
		return blastOrthology;
	}

	public void setBlastOrthology(double[][] blastOrthology) {
		this.blastOrthology = blastOrthology;
	}

	public double[][] getBitscore() {
		return bitscore;
	}

	public void setBitscore(double[][] bitscore) {
		this.bitscore = bitscore;
	}

	public double[][] getEvalue() {
		return evalue;
	}

	public void setEvalue(double[][] evalue) {
		this.evalue = evalue;
	}

	public Map<String, Integer> getGeneIndex() {
		return geneIndex;
	}

	public void setGeneIndex(Map<String, Integer> geneIndex) {
		this.geneIndex = geneIndex;
	}
	
	
	public void modify(){
		for (int i=0; i<noOfGenes; i++){
			for (int j=i+1; j<noOfGenes; j++){
				if (predictedOrthology[i][j]==0 && blastOrthology[i][j]==0 && !this.species[i].equals(this.species[j])){
					predictedOrthology[i][j]=1d;
					predictedOrthology[j][i]=1d;
				}
			}
		}
	}
	
	public String output(int index, boolean storeBitscore, boolean storeEvalue) {
		String str = "#GeneFamily " + index + "\n";
		str+="#GeneNames\n";
		for (int i=0; i<noOfGenes; i++){
			str+=this.genes[i]+"\t";
		}
		str+="\n";
		str += "#GeneSpeciesAssociations\n";
		for (int i=0; i<noOfGenes; i++){
			str+=this.species[i]+"\t";
		}
		str+="\n";
		str+="#OrthologyMatrix\n";
		for (int i=0; i<noOfGenes; i++){
			for (int j=0; j<noOfGenes; j++){
				str += predictedOrthology[i][j]<0.5?"0\t":"1\t";
			}
			str+="\n";
		}
		if (storeBitscore){
			str+="#Bitscores\n";
			for (int i=0; i<noOfGenes; i++){
				for (int j=0; j<noOfGenes; j++){
					str += String.valueOf(this.bitscore[i][j])+"\t";
				}
				str+="\n";
			}
		}
		if (storeEvalue){
			str+="#E-values\n";
			for (int i=0; i<noOfGenes; i++){
				for (int j=0; j<noOfGenes; j++){
					str += String.valueOf(this.evalue[i][j])+"\t";
				}
				str+="\n";
			}
		}
		str+="\n";
		return str;
	}
	
	private double noise(Random rand, boolean value, double noiseProbability){
		double x = rand.nextDouble();
		if (x<noiseProbability){
			return value?0d:1d;
		}
		return value?1d:0d;
	}

	public void addNoise(Random rand, int noiseType, double noiseProbability, String[] speciesNames) {
		if (noiseType>0){
			if (noiseType<4){
				for (int i=0; i<predictedOrthology.length; i++){
					for (int j=0; j<predictedOrthology[i].length; j++){
						double value = predictedOrthology[i][j];
						if (i!=j && !species[i].equals(species[j]) && ( ((noiseType==1 || noiseType==2) && value==1d) || ((noiseType==1 || noiseType==3) && value==0d) ) ){
							value = noise(rand, value>0.5, noiseProbability);
						}
						predictedOrthology[i][j]=value;
					}
				}
			}
			else if (noiseType==4){
				for (int i=0; i<species.length; i++){
					if (rand.nextDouble()<noiseProbability){
						species[i] = speciesNames[rand.nextInt(speciesNames.length)];
					}
				}
			}
		}
	}
	
	
}
