package paraphylo.converter;

import java.io.BufferedReader;
import java.io.FileReader;
import java.util.List;
import java.util.Map;

public class ProteinorthoGraphFileParser {

	public static List<GeneFamily> parse(String filename, String[] speciesNames, Map<String,Integer> geneFamilyMap, List<GeneFamily> geneFamilies) throws Exception{
		
		if (speciesNames == null) {
			throw new Exception("speciesNames has to be initialized");
		}
		if (geneFamilyMap==null){
			throw new Exception("geneFamilyMap has to be initialized");
		}
		if (geneFamilies==null){
			throw new Exception("geneFamilies has to be initialized");
		}
		
		BufferedReader br = new BufferedReader(new FileReader(filename));
		String line = "";
		while ((line = br.readLine()) != null) {
			line = line.trim();
		    if (line.startsWith("#")){
		    	continue;
		    }
		    else{
		    	String[] splitLine = line.split("\\t");
		    	String geneA = splitLine[0];
		    	Integer gfA = geneFamilyMap.get(geneA);
		    	if (gfA == null){
		    		continue;
		    	}
		    	String geneB = splitLine[1];
		    	Integer gfB = geneFamilyMap.get(geneB);
		    	if (gfB == null){
		    		continue;
		    	}
		    	if (gfA.equals(gfB)){
		    		GeneFamily geneFamily = geneFamilies.get(gfA.intValue());
		    		int a = geneFamily.getGeneIndex().get(geneA);
		    		int b = geneFamily.getGeneIndex().get(geneB);
		    		geneFamily.getPredictedOrthology()[a][b] = 1d;
		    		geneFamily.getPredictedOrthology()[b][a] = 1d;
		    		geneFamily.getBitscore()[a][b] = Double.parseDouble(splitLine[3]);
		    		geneFamily.getBitscore()[b][a] = Double.parseDouble(splitLine[5]);
		    		geneFamily.getEvalue()[a][b] = Double.parseDouble(splitLine[2]);
		    		geneFamily.getEvalue()[b][a] = Double.parseDouble(splitLine[4]);
		    	}
		    }
		}
		br.close();
		return geneFamilies;
	}
}
